@@ -29,7 +29,7 @@ static GOptionEntry entries[] =
     { "urgency",      'u', 0, G_OPTION_ARG_STRING,       &urgency_str,    "The urgency of this notification", "URG" },
     { "hints",        'h', 0, G_OPTION_ARG_STRING_ARRAY, &hint_strs,      "User specified hints", "HINT" },
     { "action",       'A', 0, G_OPTION_ARG_STRING_ARRAY, &action_strs,    "Actions the user can invoke", "ACTION" },
-    { "timeout",      't', 0, G_OPTION_ARG_INT,          &timeout,        "The time until the notification expires", "TIMEOUT" },
+    { "timeout",      't', 0, G_OPTION_ARG_INT,          &timeout,        "The time in milliseconds until the notification expires", "TIMEOUT" },
     { "icon",         'i', 0, G_OPTION_ARG_STRING,       &icon,           "An Icon that should be displayed with the notification", "ICON" },
     { "raw_icon",     'I', 0, G_OPTION_ARG_STRING,       &raw_icon_path,  "Path to the icon to be sent as raw image data", "RAW_ICON"},
     { "capabilities", 'c', 0, G_OPTION_ARG_NONE,         &capabilities,   "Print the server capabilities and exit", NULL},
@@ -81,7 +81,7 @@ void print_serverinfo(void)
  * cases. This function gets the specified argv element ignoring the first
  * terminator.
  *
- * See https://developer.gnome.org/glib/stable/glib-Commandline-option-parser.html#g-option-context-parse for details
+ * See https://docs.gtk.org/glib/method.OptionContext.parse.html for details
  */
 char *get_argv(char *argv[], int index)
 {
@@ -127,6 +127,11 @@ void parse_commandline(int argc, char *argv[])
         die(0);
     }
 
+    if (*appname == '\0') {
+        g_printerr("Provided appname was empty\n");
+        die(1);
+    }
+
     int n_args = count_args(argv, argc);
     if (n_args < 2 && close_id < 1) {
         g_printerr("I need at least a summary\n");
@@ -234,7 +239,7 @@ void add_action(NotifyNotification *n, char *str)
     char *label = strchr(str, ',');
 
     if (!label || *(label+1) == '\0') {
-        g_printerr("Malformed action. Excpected \"action,label\", got \"%s\"", str);
+        g_printerr("Malformed action. Expected \"action,label\", got \"%s\"", str);
         return;
     }
 
@@ -359,4 +364,4 @@ int main(int argc, char *argv[])
     die(0);
 }
 
-/* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
+/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
