@@ -25,6 +25,7 @@
 #include "notification.h"
 #include "settings.h"
 #include "utils.h"
+#include "output.h" // For checking if wayland is active.
 
 /* notification lists */
 static GQueue *waiting   = NULL; /**< all new notifications get into here */
@@ -51,7 +52,7 @@ GList *queues_get_displayed(void)
 }
 
 /* see queues.h */
-const struct notification *queues_get_head_waiting(void)
+struct notification *queues_get_head_waiting(void)
 {
         if (waiting->length == 0)
                 return NULL;
@@ -76,6 +77,12 @@ unsigned int queues_length_history(void)
         return history->length;
 }
 
+/* see queues.h */
+GList *queues_get_history(void)
+{
+        return g_queue_peek_head_link(history);
+}
+
 /**
  * Swap two given queue elements. The element's data has to be a notification.
  *
@@ -127,10 +134,11 @@ static bool queues_notification_is_ready(const struct notification *n, struct du
  *
  * @param n the notification to check
  * @param status the current status of dunst
+ * @param time the current time
  * @retval true: the notification is timed out
  * @retval false: otherwise
  */
-static bool queues_notification_is_finished(struct notification *n, struct dunst_status status)
+static bool queues_notification_is_finished(struct notification *n, struct dunst_status status, gint64 time)
 {
         assert(n);
 
@@ -149,7 +157,7 @@ static bool queues_notification_is_finished(struct notification *n, struct dunst
         }
 
         /* remove old message */
-        if (time_monotonic_now() - n->start > n->timeout) {
+        if (time - n->start > n->timeout) {
                 return true;
         }
 
@@ -167,19 +175,6 @@ int queues_notification_insert(struct notification *n)
                 LOG_M("Skipping notification: '%s' '%s'", n->body, n->summary);
                 return 0;
         }
-        /* Do not insert the message if it's a command */
-        if (STR_EQ("DUNST_COMMAND_PAUSE", n->summary)) {
-                dunst_status(S_RUNNING, false);
-                return 0;
-        }
-        if (STR_EQ("DUNST_COMMAND_RESUME", n->summary)) {
-                dunst_status(S_RUNNING, true);
-                return 0;
-        }
-        if (STR_EQ("DUNST_COMMAND_TOGGLE", n->summary)) {
-                dunst_status(S_RUNNING, !dunst_status_get().running);
-                return 0;
-        }
 
         bool inserted = false;
         if (n->id != 0) {
@@ -201,6 +196,10 @@ int queues_notification_insert(struct notification *n)
         if (!inserted)
                 g_queue_insert_sorted(waiting, n, notification_cmp_data, NULL);
 
+        if (!n->icon) {
+                notification_icon_replace_path(n, n->iconname);
+        }
+
         if (settings.print_notifications)
                 notification_print(n);
 
@@ -213,31 +212,33 @@ int queues_notification_insert(struct notification *n)
  * @retval true: notification got stacked
  * @retval false: notification did not get stacked
  */
-static bool queues_stack_duplicate(struct notification *n)
+static bool queues_stack_duplicate(struct notification *new)
 {
         GQueue *allqueues[] = { displayed, waiting };
         for (int i = 0; i < sizeof(allqueues)/sizeof(GQueue*); i++) {
                 for (GList *iter = g_queue_peek_head_link(allqueues[i]); iter;
                      iter = iter->next) {
-                        struct notification *orig = iter->data;
-                        if (notification_is_duplicate(orig, n)) {
+                        struct notification *old = iter->data;
+                        if (notification_is_duplicate(old, new)) {
                                 /* If the progress differs, probably notify-send was used to update the notification
                                  * So only count it as a duplicate, if the progress was not the same.
                                  * */
-                                if (orig->progress == n->progress) {
-                                        orig->dup_count++;
+                                if (old->progress == new->progress) {
+                                        old->dup_count++;
                                 } else {
-                                        orig->progress = n->progress;
+                                        old->progress = new->progress;
                                 }
-                                iter->data = n;
+                                iter->data = new;
 
-                                n->dup_count = orig->dup_count;
-                                signal_notification_closed(orig, 1);
+                                new->dup_count = old->dup_count;
+                                signal_notification_closed(old, 1);
 
                                 if (allqueues[i] == displayed)
-                                        n->start = time_monotonic_now();
+                                        new->start = time_monotonic_now();
 
-                                notification_unref(orig);
+                                notification_transfer_icon(old, new);
+
+                                notification_unref(old);
                                 return true;
                         }
                 }
@@ -259,7 +260,8 @@ static bool queues_stack_by_tag(struct notification *new)
                 for (GList *iter = g_queue_peek_head_link(allqueues[i]); iter;
                             iter = iter->next) {
                         struct notification *old = iter->data;
-                        if (STR_FULL(old->stack_tag) && STR_EQ(old->stack_tag, new->stack_tag)) {
+                        if (STR_FULL(old->stack_tag) && STR_EQ(old->stack_tag, new->stack_tag)
+                                        && STR_EQ(old->appname, new->appname)) {
                                 iter->data = new;
                                 new->dup_count = old->dup_count;
 
@@ -270,6 +272,8 @@ static bool queues_stack_by_tag(struct notification *new)
                                         notification_run_script(new);
                                 }
 
+                                notification_transfer_icon(old, new);
+
                                 notification_unref(old);
                                 return true;
                         }
@@ -349,6 +353,34 @@ void queues_history_pop(void)
         g_queue_insert_sorted(waiting, n, notification_cmp_data, NULL);
 }
 
+/* see queues.h */
+void queues_history_pop_by_id(unsigned int id)
+{
+        struct notification *n = NULL;
+
+        if (g_queue_is_empty(history))
+                return;
+
+        // search through the history buffer 
+        for (GList *iter = g_queue_peek_head_link(history); iter;
+                iter = iter->next) {
+                struct notification *cur = iter->data;
+                if (cur->id == id) {
+                        n = cur;
+                        break;
+                }
+        }
+
+        // must be a valid notification
+        if (n == NULL)
+                return;
+
+        g_queue_remove(history, n);
+        n->redisplayed = true;
+        n->timeout = settings.sticky_history ? 0 : n->timeout;
+        g_queue_insert_sorted(waiting, n, notification_cmp_data, NULL);
+}
+
 /* see queues.h */
 void queues_history_push(struct notification *n)
 {
@@ -377,7 +409,7 @@ void queues_history_push_all(void)
 }
 
 /* see queues.h */
-void queues_update(struct dunst_status status)
+void queues_update(struct dunst_status status, gint64 time)
 {
         GList *iter, *nextiter;
 
@@ -389,7 +421,20 @@ void queues_update(struct dunst_status status)
                 struct notification *n = iter->data;
                 nextiter = iter->next;
 
-                if (queues_notification_is_finished(n, status)){
+                if (notification_is_locked(n)) {
+                        iter = nextiter;
+                        continue;
+                }
+
+                if (n->marked_for_closure) {
+                        queues_notification_close(n, n->marked_for_closure);
+                        n->marked_for_closure = 0;
+                        iter = nextiter;
+                        continue;
+                }
+
+
+                if (queues_notification_is_finished(n, status, time)){
                         queues_notification_close(n, REASON_TIME);
                         iter = nextiter;
                         continue;
@@ -406,14 +451,14 @@ void queues_update(struct dunst_status status)
         }
 
         int cur_displayed_limit;
-        if (settings.geometry.h == 0)
+        if (settings.notification_limit == 0)
                 cur_displayed_limit = INT_MAX;
         else if (   settings.indicate_hidden
-                 && settings.geometry.h > 1
-                 && displayed->length + waiting->length > settings.geometry.h)
-                cur_displayed_limit = settings.geometry.h-1;
+                 && settings.notification_limit > 1
+                 && displayed->length + waiting->length > settings.notification_limit)
+                cur_displayed_limit = settings.notification_limit-1;
         else
-                cur_displayed_limit = settings.geometry.h;
+                cur_displayed_limit = settings.notification_limit;
 
         /* move notifications from queue to displayed */
         iter = g_queue_peek_head_link(waiting);
@@ -484,7 +529,7 @@ gint64 queues_get_next_datachange(gint64 time)
                 struct notification *n = iter->data;
                 gint64 ttl = n->timeout - (time - n->start);
 
-                if (n->timeout > 0) {
+                if (n->timeout > 0 && n->locked == 0) {
                         if (ttl > 0)
                                 sleep = MIN(sleep, ttl);
                         else
@@ -550,4 +595,4 @@ void queues_teardown(void)
 }
 
 
-/* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
+/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
