@@ -96,6 +96,16 @@ struct setting {
        *        .rule_offset = offsetof(struct rule, *member*);
        */
       size_t rule_offset;
+
+      /**
+       * True if a setting has a different default in the default dunstrc.
+       * This is useful to transition a default value without breaking exisitng
+       * configs. This value is needed for the test suite to skip testing this
+       * setting against the default dunstrc.
+       *
+       * False by default.
+       */
+      bool different_default;
 };
 
 
@@ -125,19 +135,21 @@ struct setting {
 static const struct rule empty_rule = {
         .name            = "empty",
         .appname         = NULL,
+        .action_name     = NULL,
         .summary         = NULL,
         .body            = NULL,
         .icon            = NULL,
         .category        = NULL,
         .msg_urgency     = URG_NONE,
+        .match_dbus_timeout = -1,
         .timeout         = -1,
+        .override_dbus_timeout       = -1,
         .urgency         = URG_NONE,
         .markup          = MARKUP_NULL,
         .history_ignore  = -1,
         .match_transient = -1,
         .set_transient   = -1,
         .icon_position   = -1,
-        .set_icon_size   = -1,
         .skip_display    = -1,
         .word_wrap       = -1,
         .ellipsize       = -1,
@@ -151,6 +163,8 @@ static const struct rule empty_rule = {
         .script          = NULL,
         .enabled         = true,
         .progress_bar_alignment   = -1,
+        .min_icon_size   = -1,
+        .max_icon_size   = -1,
 };
 
 
@@ -414,6 +428,17 @@ static const struct setting allowed_settings[] = {
                 .parser_data = urgency_enum_data,
                 .rule_offset = offsetof(struct rule, msg_urgency),
         },
+        {
+                .name = "match_dbus_timeout",
+                .section = "*",
+                .description = "Matches the dbus_timeout of the notification",
+                .type = TYPE_TIME,
+                .default_value = "*",
+                .value = NULL,
+                .parser = NULL,
+                .parser_data = NULL,
+                .rule_offset = offsetof(struct rule, match_dbus_timeout),
+        },
         {
                 .name = "stack_tag",
                 .section = "*",
@@ -460,6 +485,17 @@ static const struct setting allowed_settings[] = {
                 .parser_data = NULL,
                 .rule_offset = offsetof(struct rule, bg),
         },
+        {
+                .name = "action_name",
+                .section = "*",
+                .description = "Sets the name of the action to be invoked on do_action.",
+                .type = TYPE_STRING,
+                .default_value = "*",
+                .value = NULL,
+                .parser = NULL,
+                .parser_data = NULL,
+                .rule_offset = offsetof(struct rule, action_name),
+        },
         {
                 .name = "foreground",
                 .section = "*",
@@ -570,6 +606,17 @@ static const struct setting allowed_settings[] = {
                 .parser_data = NULL,
                 .rule_offset = offsetof(struct rule, timeout),
         },
+        {
+                .name = "override_dbus_timeout",
+                .section = "*",
+                .description = "Replace the dbus timeout with this value.",
+                .type = TYPE_TIME,
+                .default_value = "*",
+                .value = NULL,
+                .parser = NULL,
+                .parser_data = NULL,
+                .rule_offset = offsetof(struct rule, override_dbus_timeout),
+        },
         {
                 .name = "urgency",
                 .section = "*",
@@ -669,17 +716,6 @@ static const struct setting allowed_settings[] = {
                 .parser_data = icon_position_enum_data,
                 .rule_offset = offsetof(struct rule, icon_position),
         },
-        {
-                .name = "icon_size",
-                .section = "*",
-                .description = "Set the size of the icon",
-                .type = TYPE_INT,
-                .default_value = "*",
-                .value = NULL,
-                .parser = NULL,
-                .parser_data = NULL,
-                .rule_offset = offsetof(struct rule, set_icon_size),
-        },
         {
                 .name = "enabled",
                 .section = "*",
@@ -702,6 +738,28 @@ static const struct setting allowed_settings[] = {
                 .parser_data = horizontal_alignment_enum_data,
                 .rule_offset = offsetof(struct rule, progress_bar_alignment),
         },
+        {
+                .name = "min_icon_size",
+                .section = "global",
+                .description = "Scale smaller icons up to this size, set to 0 to disable. If max_icon_size also specified, that has the final say.",
+                .type = TYPE_INT,
+                .default_value = "*",
+                .value = NULL,
+                .parser = NULL,
+                .parser_data = NULL,
+                .rule_offset = offsetof(struct rule, min_icon_size),
+        },
+        {
+                .name = "max_icon_size",
+                .section = "global",
+                .description = "Scale larger icons down to this size, set to 0 to disable",
+                .type = TYPE_INT,
+                .default_value = "*",
+                .value = NULL,
+                .parser = NULL,
+                .parser_data = NULL,
+                .rule_offset = offsetof(struct rule, max_icon_size),
+        },
         // end of modifying rules
 
         // other settings below
@@ -1046,26 +1104,6 @@ static const struct setting allowed_settings[] = {
                 .parser = NULL,
                 .parser_data = &settings.browser_cmd,
         },
-        {
-                .name = "min_icon_size",
-                .section = "global",
-                .description = "Scale smaller icons up to this size, set to 0 to disable. If max_icon_size also specified, that has the final say.",
-                .type = TYPE_INT,
-                .default_value = "0",
-                .value = &settings.min_icon_size,
-                .parser = NULL,
-                .parser_data = NULL,
-        },
-        {
-                .name = "max_icon_size",
-                .section = "global",
-                .description = "Scale larger icons down to this size, set to 0 to disable",
-                .type = TYPE_INT,
-                .default_value = "32",
-                .value = &settings.max_icon_size,
-                .parser = NULL,
-                .parser_data = NULL,
-        },
         {
                 .name = "always_run_script",
                 .section = "global",
@@ -1186,6 +1224,7 @@ static const struct setting allowed_settings[] = {
                 .value = &settings.enable_recursive_icon_lookup,
                 .parser = string_parse_bool,
                 .parser_data = boolean_enum_data,
+                .different_default = true,
         },
         {
                 .name = "enable_posix_regex",
@@ -1453,6 +1492,16 @@ static const struct setting allowed_settings[] = {
                 .parser = NULL,
                 .parser_data = NULL,
         },
+        {
+                .name = "gap_size",
+                .section = "global",
+                .description = "Size of gap between notifications",
+                .type = TYPE_INT,
+                .default_value = "0",
+                .value = &settings.gap_size,
+                .parser = NULL,
+                .parser_data = NULL,
+        },
 };
 #endif
 /* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
