@@ -19,6 +19,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xresource.h>
 #include <X11/Xutil.h>
+#include <linux/input-event-codes.h>
 
 #include "../dbus.h"
 #include "../draw.h"
@@ -30,6 +31,7 @@
 #include "../queues.h"
 #include "../settings.h"
 #include "../utils.h"
+#include "../input.h"
 
 #include "screen.h"
 
@@ -67,8 +69,10 @@ static int x_shortcut_tear_down_error_handler(void);
 static void setopacity(Window win, unsigned long opacity);
 static void x_handle_click(XEvent ev);
 
-static void x_win_move(struct window_x11 *win, int x, int y, int width, int height)
+static void x_win_move(window winptr, int x, int y, int width, int height)
 {
+        struct window_x11 *win = (struct window_x11*)winptr;
+
         /* move and resize */
         if (x != win->dim.x || y != win->dim.y) {
                 XMoveWindow(xctx.dpy, win->xwin, x, y);
@@ -109,7 +113,7 @@ static void x_win_corners_shape(struct window_x11 *win, const int rad)
 
         draw_rounded_rect(cr, 0, 0,
                           width, height,
-                          rad,
+                          rad, 1,
                           true, true);
         cairo_fill(cr);
 
@@ -126,8 +130,9 @@ static void x_win_corners_shape(struct window_x11 *win, const int rad)
                 win->xwin, ShapeNotifyMask);
 }
 
-static void x_win_corners_unshape(struct window_x11 *win)
+static void x_win_corners_unshape(window winptr)
 {
+        struct window_x11 *win = (struct window_x11*)winptr;
         XRectangle rect = {
                 .x = 0,
                 .y = 0,
@@ -153,10 +158,17 @@ static bool x_win_composited(struct window_x11 *win)
         }
 }
 
-void x_display_surface(cairo_surface_t *srf, struct window_x11 *win, const struct dimensions *dim)
+void x_display_surface(cairo_surface_t *srf, window winptr, const struct dimensions *dim)
 {
-        x_win_move(win, dim->x, dim->y, dim->w, dim->h);
-        cairo_xlib_surface_set_size(win->root_surface, dim->w, dim->h);
+        struct window_x11 *win = (struct window_x11*)winptr;
+        const struct screen_info *scr = get_active_screen();
+        double scale = x_get_scale();
+        int x, y;
+
+        calc_window_pos(scr, round(dim->w * scale), round(dim->h * scale), &x, &y);
+
+        x_win_move(win, x, y, round(dim->w * scale), round(dim->h * scale));
+        cairo_xlib_surface_set_size(win->root_surface, round(dim->w * scale), round(dim->h * scale));
 
         XClearWindow(xctx.dpy, win->xwin);
 
@@ -165,7 +177,7 @@ void x_display_surface(cairo_surface_t *srf, struct window_x11 *win, const struc
         cairo_show_page(win->c_ctx);
 
         if (settings.corner_radius != 0 && ! x_win_composited(win))
-                x_win_corners_shape(win, dim->corner_radius);
+                x_win_corners_shape(win, round(dim->corner_radius * scale));
         else
                 x_win_corners_unshape(win);
 
@@ -173,14 +185,9 @@ void x_display_surface(cairo_surface_t *srf, struct window_x11 *win, const struc
 
 }
 
-bool x_win_visible(struct window_x11 *win)
+cairo_t* x_win_get_context(window winptr)
 {
-        return win->visible;
-}
-
-cairo_t* x_win_get_context(struct window_x11 *win)
-{
-        return win->c_ctx;
+        return ((struct window_x11*)win)->c_ctx;
 }
 
 static void setopacity(Window win, unsigned long opacity)
@@ -278,7 +285,7 @@ gboolean x_mainloop_fd_dispatch(GSource *source, GSourceFunc callback, gpointer
         struct window_x11 *win = ((struct x11_source*) source)->win;
 
         bool fullscreen_now;
-        struct screen_info *scr;
+        const struct screen_info *scr;
         XEvent ev;
         unsigned int state;
         while (XPending(xctx.dpy) > 0) {
@@ -399,59 +406,49 @@ bool x_is_idle(void)
         return xctx.screensaver_info->idle > settings.idle_threshold / 1000;
 }
 
+/*
+ * Convert x button code to linux event code
+ * Returns 0 if button is not recognized.
+ */
+static unsigned int x_mouse_button_to_linux_event_code(unsigned int x_button)
+{
+        switch (x_button) {
+                case Button1:
+                        return BTN_LEFT;
+                case Button2:
+                        return BTN_MIDDLE;
+                case Button3:
+                        return BTN_RIGHT;
+                default:
+                        LOG_W("Unsupported mouse button: '%d'", x_button);
+                        return 0;
+        }
+}
+
 /* TODO move to x_mainloop_* */
 /*
  * Handle incoming mouse click events
  */
 static void x_handle_click(XEvent ev)
 {
-        enum mouse_action *acts;
+        unsigned int linux_code = x_mouse_button_to_linux_event_code(ev.xbutton.button);
 
-        switch (ev.xbutton.button) {
-                case Button1:
-                        acts = settings.mouse_left_click;
-                        break;
-                case Button2:
-                        acts = settings.mouse_middle_click;
-                        break;
-                case Button3:
-                        acts = settings.mouse_right_click;
-                        break;
-                default:
-                        LOG_W("Unsupported mouse button: '%d'", ev.xbutton.button);
-                        return;
+        if (linux_code == 0) {
+                return;
         }
 
-        for (int i = 0; acts[i]; i++) {
-                enum mouse_action act = acts[i];
-                if (act == MOUSE_CLOSE_ALL) {
-                        queues_history_push_all();
-                        return;
-                }
-
-                if (act == MOUSE_DO_ACTION || act == MOUSE_CLOSE_CURRENT) {
-                        int y = settings.separator_height;
-                        struct notification *n = NULL;
-                        int first = true;
-                        for (const GList *iter = queues_get_displayed(); iter;
-                             iter = iter->next) {
-                                n = iter->data;
-                                if (ev.xbutton.y > y && ev.xbutton.y < y + n->displayed_height)
-                                        break;
-
-                                y += n->displayed_height + settings.separator_height;
-                                if (first)
-                                        y += settings.frame_width;
-                        }
-
-                        if (n) {
-                                if (act == MOUSE_CLOSE_CURRENT)
-                                        queues_notification_close(n, REASON_USER);
-                                else
-                                        notification_do_action(n);
-                        }
-                }
+        bool button_state;
+        if(ev.type == ButtonRelease) {
+                button_state = false; // button is up
+        } else {
+                // this shouldn't happen, because this function
+                // is only called when it'a a ButtonRelease event
+                button_state = true; // button is down
         }
+
+        float scale = x_get_scale();
+        input_handle_click(linux_code, button_state, ev.xbutton.x/scale,
+                        ev.xbutton.y/scale);
 }
 
 void x_free(void)
@@ -507,14 +504,16 @@ static void XRM_update_db(void)
 /*
  * Setup X11 stuff
  */
-void x_setup(void)
+bool x_setup(void)
 {
 
         /* initialize xctx.dc, font, keyboard, colors */
         if (!setlocale(LC_CTYPE, "") || !XSupportsLocale())
                 LOG_W("No locale support");
+
         if (!(xctx.dpy = XOpenDisplay(NULL))) {
-                DIE("Cannot open X11 display.");
+                LOG_W("Cannot open X11 display.");
+                return false;
         }
 
         x_shortcut_init(&settings.close_ks);
@@ -533,32 +532,12 @@ void x_setup(void)
 
         xctx.screensaver_info = XScreenSaverAllocInfo();
 
+        XrmInitialize();
+        XRM_update_db();
+
         init_screens();
         x_shortcut_grab(&settings.history_ks);
-
-        XrmInitialize();
-}
-
-struct geometry x_parse_geometry(const char *geom_str)
-{
-        assert(geom_str);
-        struct geometry geometry = { 0 };
-
-        if (geom_str[0] == '-') {
-                geometry.negative_width = true;
-                geom_str++;
-        } else {
-                geometry.negative_width = false;
-        }
-
-        int mask = XParseGeometry(geom_str,
-                                  &geometry.x, &geometry.y,
-                                  &geometry.w, &geometry.h);
-        geometry.width_set = mask & WidthValue;
-        geometry.negative_x = mask & XNegative;
-        geometry.negative_y = mask & YNegative;
-
-        return geometry;
+        return true;
 }
 
 static void x_set_wm(Window win)
@@ -641,7 +620,7 @@ GSource* x_win_reg_source(struct window_x11 *win)
 /*
  * Setup the window
  */
-struct window_x11 *x_win_create(void)
+window x_win_create(void)
 {
         struct window_x11 *win = g_malloc0(sizeof(struct window_x11));
 
@@ -671,7 +650,7 @@ struct window_x11 *x_win_create(void)
             ExposureMask | KeyPressMask | VisibilityChangeMask |
             ButtonReleaseMask | FocusChangeMask| StructureNotifyMask;
 
-        struct screen_info *scr = get_active_screen();
+        const struct screen_info *scr = get_active_screen();
         win->xwin = XCreateWindow(xctx.dpy,
                                  root,
                                  scr->x,
@@ -713,11 +692,13 @@ struct window_x11 *x_win_create(void)
         }
         XSelectInput(xctx.dpy, root, root_event_mask);
 
-        return win;
+        return (window)win;
 }
 
-void x_win_destroy(struct window_x11 *win)
+void x_win_destroy(window winptr)
 {
+        struct window_x11 *win = (struct window_x11*)winptr;
+
         g_source_destroy(win->esrc);
         g_source_unref(win->esrc);
 
@@ -731,12 +712,15 @@ void x_win_destroy(struct window_x11 *win)
 /*
  * Show the window and grab shortcuts.
  */
-void x_win_show(struct window_x11 *win)
+void x_win_show(window winptr)
 {
+        struct window_x11 *win = (struct window_x11*)winptr;
+
         /* window is already mapped or there's nothing to show */
         if (win->visible)
                 return;
 
+
         x_shortcut_grab(&settings.close_ks);
         x_shortcut_grab(&settings.close_all_ks);
         x_shortcut_grab(&settings.context_ks);
@@ -758,13 +742,17 @@ void x_win_show(struct window_x11 *win)
 
         XMapRaised(xctx.dpy, win->xwin);
         win->visible = true;
+
+        x_display_surface(win->root_surface, win, &win->dim);
 }
 
 /*
  * Hide the window and ungrab unused keyboard_shortcuts
  */
-void x_win_hide(struct window_x11 *win)
+void x_win_hide(window winptr)
 {
+        LOG_I("X11: Hiding window");
+        struct window_x11 *win = (struct window_x11*)winptr;
         ASSERT_OR_RET(win->visible,);
 
         x_shortcut_ungrab(&settings.close_ks);
@@ -940,4 +928,15 @@ static void x_shortcut_init(struct keyboard_shortcut *ks)
         g_free(str_begin);
 }
 
-/* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
+double x_get_scale(void) {
+        if (settings.scale > 0)
+                return settings.scale;
+
+        const struct screen_info *scr_info = get_active_screen();
+        double scale = MAX(1, scr_info->dpi/96.);
+        LOG_D("X11 dpi: %i", scr_info->dpi);
+        LOG_D("X11 scale: %f", scale);
+        return scale;
+}
+
+/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
