@@ -1,5 +1,6 @@
 #define wake_up wake_up_void
 #include "../src/dbus.c"
+#include "../src/rules.h"
 #include "greatest.h"
 
 #include <assert.h>
@@ -408,11 +409,11 @@ TEST test_hint_transient(void)
                 ASSERT(dbus_notification_fire(n_dbus, &id));
                 ASSERT(id != 0);
 
-                ASSERT_EQ(queues_length_waiting(), len+1);
+                snprintf(msg, sizeof(msg), "In round %ld", i);
+                ASSERT_EQm(msg, queues_length_waiting(), len+1);
 
                 n = queues_debug_find_notification_by_id(id);
 
-                snprintf(msg, sizeof(msg), "In round %ld", i);
                 ASSERT_EQm(msg, values[i], n->transient);
         }
 
@@ -453,11 +454,12 @@ TEST test_hint_progress(void)
                 ASSERT(dbus_notification_fire(n_dbus, &id));
                 ASSERT(id != 0);
 
-                ASSERT_EQ(queues_length_waiting(), len+1);
+                snprintf(msg, sizeof(msg), "In round %ld", i);
+                ASSERT_EQm(msg, queues_length_waiting(), len+1);
 
                 n = queues_debug_find_notification_by_id(id);
 
-                snprintf(msg, sizeof(msg), "In round %ld", i);
+                snprintf(msg, sizeof(msg), "In round %ld progress should be %i, but is %i", i, n->progress, values[i]);
                 ASSERT_EQm(msg, values[i], n->progress);
         }
 
@@ -675,7 +677,11 @@ TEST test_server_caps(enum markup_mode markup)
         GVariant *caps = NULL;
         const char **capsarray;
 
-        settings.markup = markup;
+        struct rule *global_rule = get_rule("global");
+        if (!global_rule) {
+                global_rule = rule_new("global");
+        }
+        global_rule->markup = markup;
 
         reply = dbus_invoke("GetCapabilities", NULL);
 
@@ -686,9 +692,10 @@ TEST test_server_caps(enum markup_mode markup)
         ASSERT(g_strv_contains(capsarray, "actions"));
         ASSERT(g_strv_contains(capsarray, "body"));
         ASSERT(g_strv_contains(capsarray, "body-hyperlinks"));
+        ASSERT(g_strv_contains(capsarray, "icon-static"));
         ASSERT(g_strv_contains(capsarray, "x-dunst-stack-tag"));
 
-        if (settings.markup != MARKUP_NO)
+        if (markup != MARKUP_NO)
                 ASSERT(g_strv_contains(capsarray, "body-markup"));
         else
                 ASSERT_FALSE(g_strv_contains(capsarray, "body-markup"));
@@ -808,6 +815,91 @@ TEST assert_methodlists_sorted(void)
         PASS();
 }
 
+TEST test_override_dbus_timeout(void)
+{
+        struct notification *n;
+        struct dbus_notification *n_dbus;
+        struct rule *rule;
+
+        n_dbus = dbus_notification_new();
+        n_dbus->app_name = "dunstteststack";
+        n_dbus->expire_timeout = 2147484;
+
+        rule = rule_new("test_override_dbus_timeout");
+        rule->appname = "dunstteststack";
+        rule->override_dbus_timeout = 100000;
+
+        gint64 expected_timeout = rule->override_dbus_timeout;
+
+        guint id;
+        ASSERT(dbus_notification_fire(n_dbus, &id));
+        ASSERT(id != 0);
+
+        n = queues_debug_find_notification_by_id(id);
+        ASSERT_EQ_FMT(expected_timeout, n->timeout, "%" G_GINT64_FORMAT);
+
+        dbus_notification_free(n_dbus);
+        rule->enabled = false;
+
+        PASS();
+}
+
+TEST test_match_dbus_timeout(void)
+{
+        struct notification *n;
+        struct dbus_notification *n_dbus;
+        struct rule *rule;
+
+        n_dbus = dbus_notification_new();
+        n_dbus->app_name = "dunstteststack";
+        n_dbus->expire_timeout = 2147484;
+
+        rule = rule_new("test_match_dbus_timeout");
+        rule->match_dbus_timeout = 2147484000;
+        rule->override_dbus_timeout = 100000;
+
+        gint64 expected_timeout = rule->override_dbus_timeout;
+
+        guint id;
+        ASSERT(dbus_notification_fire(n_dbus, &id));
+        ASSERT(id != 0);
+
+        n = queues_debug_find_notification_by_id(id);
+        ASSERT_EQ_FMT(expected_timeout, n->timeout, "%" G_GINT64_FORMAT);
+
+        dbus_notification_free(n_dbus);
+        rule->enabled = false;
+
+        PASS();
+}
+
+TEST test_timeout(void)
+{
+        struct notification *n;
+        struct dbus_notification *n_dbus;
+        struct rule *rule;
+
+        n_dbus = dbus_notification_new();
+        n_dbus->app_name = "dunstteststack";
+
+        rule = rule_new("test_timeout");
+        rule->appname = "dunstteststack";
+        rule->timeout = 100001;
+
+        gint64 expected_timeout = rule->timeout;
+
+        guint id;
+        ASSERT(dbus_notification_fire(n_dbus, &id));
+        ASSERT(id != 0);
+
+        n = queues_debug_find_notification_by_id(id);
+        ASSERT_EQ_FMT(expected_timeout, n->timeout, "%" G_GINT64_FORMAT);
+
+        dbus_notification_free(n_dbus);
+        rule->enabled = false;
+
+        PASS();
+}
 
 // TESTS END
 
@@ -837,6 +929,9 @@ gpointer run_threaded_tests(gpointer data)
         RUN_TEST(test_close_and_signal);
         RUN_TEST(test_signal_actioninvoked);
         RUN_TEST(test_timeout_overflow);
+        RUN_TEST(test_override_dbus_timeout);
+        RUN_TEST(test_match_dbus_timeout);
+        RUN_TEST(test_timeout);
 
         RUN_TEST(assert_methodlists_sorted);
 
@@ -847,8 +942,6 @@ gpointer run_threaded_tests(gpointer data)
 
 SUITE(suite_dbus)
 {
-        settings.icon_path = "";
-
         GTestDBus *dbus_bus;
         g_test_dbus_unset();
         queues_init();
@@ -856,6 +949,10 @@ SUITE(suite_dbus)
         loop = g_main_loop_new(NULL, false);
 
         dbus_bus = g_test_dbus_new(G_TEST_DBUS_NONE);
+
+        // workaround bug in glib where stdout output is duplicated
+        // See https://gitlab.gnome.org/GNOME/glib/-/issues/2322
+        fflush(stdout);
         g_test_dbus_up(dbus_bus);
 
         thread_tests = g_thread_new("testexecutor", run_threaded_tests, loop);
@@ -866,8 +963,6 @@ SUITE(suite_dbus)
         g_object_unref(dbus_bus);
         g_thread_unref(thread_tests);
         g_main_loop_unref(loop);
-
-        settings.icon_path = NULL;
 }
 
 /* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
