@@ -5,6 +5,7 @@
 
 #include "greatest.h"
 #include "queues.h"
+#include "helpers.h"
 
 struct notification *queues_debug_find_notification_by_id(int id)
 {
@@ -35,7 +36,7 @@ TEST test_queue_length(void)
 
         n = test_notification("n2", 0);
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
 
         n = test_notification("n3", 0);
         queues_notification_insert(n);
@@ -112,7 +113,7 @@ TEST test_queue_insert_id_replacement(void)
         ASSERT_EQ(a->id, b->id);
         NOT_LAST(a);
 
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         c = test_notification("c", -1);
         c->id = b->id;
 
@@ -137,7 +138,7 @@ TEST test_queue_notification_close(void)
         queues_notification_insert(n);
         QUEUE_LEN_ALL(1, 0, 0);
         queues_notification_close(n, REASON_UNDEF);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 0, 1);
         queues_teardown();
 
@@ -147,10 +148,10 @@ TEST test_queue_notification_close(void)
         queues_init();
         queues_notification_insert(n);
         QUEUE_LEN_ALL(1, 0, 0);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 1, 0);
         queues_notification_close(n, REASON_UNDEF);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 0, 1);
         queues_teardown();
 
@@ -169,7 +170,7 @@ TEST test_queue_notification_close_histignore(void)
         queues_notification_insert(n);
         QUEUE_LEN_ALL(1, 0, 0);
         queues_notification_close(n, REASON_UNDEF);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 0, 0);
         queues_teardown();
 
@@ -180,10 +181,10 @@ TEST test_queue_notification_close_histignore(void)
         queues_init();
         queues_notification_insert(n);
         QUEUE_LEN_ALL(1, 0, 0);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 1, 0);
         queues_notification_close(n, REASON_UNDEF);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 0, 0);
         queues_teardown();
 
@@ -201,7 +202,7 @@ TEST test_queue_notification_skip_display(void)
         queues_init();
         queues_notification_insert(n);
         QUEUE_LEN_ALL(1, 0, 0);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 0, 1);
         queues_teardown();
 
@@ -219,12 +220,12 @@ TEST test_queue_notification_skip_display_redisplayed(void)
         queues_init();
         queues_notification_insert(n);
         QUEUE_LEN_ALL(1, 0, 0);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0, 0, 1);
 
         queues_history_pop();
         QUEUE_LEN_ALL(1, 0, 0);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_CONTAINSm("A skip display notification should stay in displayed "
                         "queue when it got pulled out of history queue",
                         DISP, n);
@@ -234,6 +235,72 @@ TEST test_queue_notification_skip_display_redisplayed(void)
         PASS();
 }
 
+TEST test_queue_notification_skip_display_redisplayed_by_random_id(void) {
+        // breaks if the notification_buffer_size is above 5,
+        // possible bug??????
+        size_t notification_buffer_size = 5;
+
+        struct notification *n[notification_buffer_size];
+        bool is_n_popped[notification_buffer_size];
+
+        // insert notifications
+        char string[128];
+        for(size_t i=0; i<notification_buffer_size; i++) {
+                snprintf(string, 128, "n%08lu", i+1);
+
+                n[i] = test_notification(string, -1);
+                n[i]->skip_display = true;
+                n[i]->id = i+1;
+
+                // generate a noisy pop table
+                is_n_popped[i] = rand() > RAND_MAX/4;
+        }
+
+        queues_init();
+        for(size_t i=0; i<notification_buffer_size; i++) {
+                queues_notification_insert(n[i]);
+        }
+
+        QUEUE_LEN_ALL(notification_buffer_size, 0, 0);
+
+        // update notification event
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+
+        QUEUE_LEN_ALL(0, 0, notification_buffer_size);
+
+        // pop according to the pop table
+        size_t popped_notification_count = 0;
+        for(size_t i=0; i<notification_buffer_size; i++) {
+                if(is_n_popped[i]) {
+                        queues_history_pop_by_id(i+1);
+                        popped_notification_count++;
+                        queues_update(STATUS_NORMAL, time_monotonic_now());
+                }
+        }
+
+        // test whether the notifications were unpopped properly
+        QUEUE_LEN_ALL(0, popped_notification_count,
+                        notification_buffer_size-popped_notification_count);
+
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+        // check if any of the notifications got moved
+        for(size_t i=0; i<notification_buffer_size; i++) {
+                if(is_n_popped[i]) {
+                        QUEUE_CONTAINSm("A skip display notification should stay in displayed "
+                                        "queue when it got pulled out of history queue",
+                                        DISP, n[i]);
+                } else {
+                        QUEUE_CONTAINSm("A skip display notification that didn't get popped"
+                                        "out of history should've stayed there",
+                                        HIST, n[i]);
+                }
+        }
+
+        queues_teardown();
+
+        PASS();
+}
+
 TEST test_queue_history_overfull(void)
 {
         settings.history_length = 10;
@@ -245,7 +312,7 @@ TEST test_queue_history_overfull(void)
                 char name[] = { 'n', '0'+i, '\0' }; // n<i>
                 n = test_notification(name, -1);
                 queues_notification_insert(n);
-                queues_update(STATUS_NORMAL);
+                queues_update(STATUS_NORMAL, time_monotonic_now());
                 queues_notification_close(n, REASON_UNDEF);
         }
 
@@ -266,7 +333,7 @@ TEST test_queue_history_pushall(void)
 {
         settings.history_length = 5;
         settings.indicate_hidden = false;
-        settings.geometry.h = 0;
+        settings.notification_limit = 0;
 
         queues_init();
 
@@ -277,7 +344,7 @@ TEST test_queue_history_pushall(void)
                 n = test_notification(name, -1);
                 queues_notification_insert(n);
         }
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
 
         for (int i = 0; i < 10; i++) {
                 char name[] = { '2', 'n', '0'+i, '\0' }; // 2n<i>
@@ -343,7 +410,7 @@ TEST test_datachange_endless(void)
         struct notification *n = test_notification("n", 0);
 
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
 
         ASSERTm("Age threshold is deactivated and the notification is infinite, there is no wakeup necessary.",
                 queues_get_next_datachange(time_monotonic_now()) < 0);
@@ -361,7 +428,7 @@ TEST test_datachange_endless_agethreshold(void)
         struct notification *n = test_notification("n", 0);
 
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
 
         ASSERT_IN_RANGEm("Age threshold is activated and the next wakeup should be less than a second away",
                 S2US(1)/2, queues_get_next_datachange(time_monotonic_now() + S2US(4)), S2US(1)/2);
@@ -387,7 +454,7 @@ TEST test_datachange_queues(void)
         ASSERTm("The inserted notification is inside the waiting queue, so it should get ignored.",
                queues_get_next_datachange(time_monotonic_now()) < S2US(0));
 
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         ASSERT_IN_RANGEm("The notification has to get closed in less than its timeout",
                S2US(10)/2, queues_get_next_datachange(time_monotonic_now()), S2US(10)/2);
 
@@ -407,14 +474,14 @@ TEST test_datachange_ttl(void)
         n = test_notification("n1", 15);
 
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         ASSERT_IN_RANGEm("The notification has to get closed in less than its timeout.",
                n->timeout/2, queues_get_next_datachange(time_monotonic_now()), n->timeout/2);
 
         n = test_notification("n2", 10);
 
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         ASSERT_IN_RANGEm("The timeout of the second notification has to get used as sleep time now.",
                n->timeout/2, queues_get_next_datachange(time_monotonic_now()), n->timeout/2);
 
@@ -444,7 +511,7 @@ TEST test_queue_stacking(void)
         NOT_LAST(n1);
 
         notification_ref(n2);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         queues_notification_insert(n3);
         QUEUE_LEN_ALL(0, 1, 0);
         NOT_LAST(n2);
@@ -461,8 +528,8 @@ TEST test_queue_stacktag(void)
         queues_init();
 
         n1 = test_notification("n1", 1);
-        n2 = test_notification("n2", 1);
-        n3 = test_notification("n3", 1);
+        n2 = test_notification("n1", 1);
+        n3 = test_notification("n1", 1);
         n1->stack_tag = g_strdup(stacktag);
         n2->stack_tag = g_strdup(stacktag);
         n3->stack_tag = g_strdup(stacktag);
@@ -475,7 +542,7 @@ TEST test_queue_stacktag(void)
         NOT_LAST(n1);
 
         notification_ref(n2);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         queues_notification_insert(n3);
         QUEUE_LEN_ALL(0, 1, 0);
         NOT_LAST(n2);
@@ -484,9 +551,70 @@ TEST test_queue_stacktag(void)
         PASS();
 }
 
+TEST test_queue_different_stacktag(void)
+{
+        const char *stacktag = "asdf";
+        const char *stacktag2 = "something else";
+        struct notification *n1, *n2, *n3;
+        settings.stack_duplicates = false;
+
+        queues_init();
+
+        n1 = test_notification("n1", 1);
+        n2 = test_notification("n1", 1);
+        n3 = test_notification("n1", 1);
+        n1->stack_tag = g_strdup(stacktag);
+        n2->stack_tag = g_strdup(stacktag);
+        n3->stack_tag = g_strdup(stacktag2);
+
+        queues_notification_insert(n1);
+        QUEUE_LEN_ALL(1, 0, 0);
+
+        notification_ref(n1);
+        queues_notification_insert(n2);
+        NOT_LAST(n1);
+
+        queues_notification_insert(n3);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+        printf("queue %i\n",g_queue_get_length(QUEUE(HIST)));
+        QUEUE_LEN_ALL(0, 2, 0);
+
+        queues_teardown();
+        PASS();
+}
+
+TEST test_queue_stacktag_different_appid(void)
+{
+        const char *stacktag = "THIS IS A SUPER WIERD STACK TAG";
+        struct notification *n1, *n2, *n3;
+
+        queues_init();
+
+        n1 = test_notification("n1", 1);
+        n2 = test_notification("n2", 1);
+        n3 = test_notification("n2", 1);
+        n1->stack_tag = g_strdup(stacktag);
+        n2->stack_tag = g_strdup(stacktag);
+        n3->stack_tag = g_strdup(stacktag);
+
+        queues_notification_insert(n1);
+        QUEUE_LEN_ALL(1, 0, 0);
+
+        queues_notification_insert(n2);
+
+        notification_ref(n2);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+        queues_notification_insert(n3);
+        QUEUE_LEN_ALL(0, 2, 0);
+        NOT_LAST(n2);
+
+        queues_teardown();
+        PASS();
+}
+
 TEST test_queue_timeout(void)
 {
-        settings.geometry.h = 5;
+        settings.notification_limit = 5;
         struct notification *n1, *n2, *n3;
 
         queues_init();
@@ -500,13 +628,13 @@ TEST test_queue_timeout(void)
         queues_notification_insert(n2);
         queues_notification_insert(n3);
 
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
 
         // hacky way to shift time
         n1->start -= S2US(11);
         n2->start -= S2US(11);
         n3->start -= S2US(11);
-        queues_update(STATUS_IDLE);
+        queues_update(STATUS_IDLE, time_monotonic_now());
 
         QUEUE_LEN_ALL(0,2,1);
         QUEUE_CONTAINS(HIST, n3);
@@ -514,7 +642,7 @@ TEST test_queue_timeout(void)
         // hacky way to shift time
         n1->start -= S2US(11);
         n2->start -= S2US(11);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
 
         QUEUE_LEN_ALL(0,1,2);
         QUEUE_CONTAINS(DISP, n1);
@@ -527,7 +655,7 @@ TEST test_queue_timeout(void)
 
 TEST test_queues_update_fullscreen(void)
 {
-        settings.geometry.h = 5;
+        settings.notification_limit = 5;
         struct notification *n_show, *n_dela, *n_push;
 
         queues_init();
@@ -543,17 +671,17 @@ TEST test_queues_update_fullscreen(void)
         queues_notification_insert(n_dela);
         queues_notification_insert(n_push);
 
-        queues_update(STATUS_FS);
+        queues_update(STATUS_FS, time_monotonic_now());
         QUEUE_CONTAINS(DISP, n_show);
         QUEUE_CONTAINS(WAIT, n_dela);
         QUEUE_CONTAINS(WAIT, n_push);
 
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_CONTAINS(DISP, n_show);
         QUEUE_CONTAINS(DISP, n_dela);
         QUEUE_CONTAINS(DISP, n_push);
 
-        queues_update(STATUS_FS);
+        queues_update(STATUS_FS, time_monotonic_now());
         QUEUE_CONTAINS(DISP, n_show);
         QUEUE_CONTAINS(DISP, n_dela);
         QUEUE_CONTAINS(WAIT, n_push);
@@ -564,7 +692,7 @@ TEST test_queues_update_fullscreen(void)
 
 TEST test_queues_update_paused(void)
 {
-        settings.geometry.h = 5;
+        settings.notification_limit = 5;
         struct notification *n1, *n2, *n3;
         queues_init();
 
@@ -578,13 +706,13 @@ TEST test_queues_update_paused(void)
 
         QUEUE_LEN_ALL(3,0,0);
 
-        queues_update(STATUS_PAUSE);
+        queues_update(STATUS_PAUSE, time_monotonic_now());
         QUEUE_LEN_ALL(3,0,0);
 
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0,3,0);
 
-        queues_update(STATUS_PAUSE);
+        queues_update(STATUS_PAUSE, time_monotonic_now());
         QUEUE_LEN_ALL(3,0,0);
 
         queues_teardown();
@@ -593,7 +721,7 @@ TEST test_queues_update_paused(void)
 
 TEST test_queues_update_seeping(void)
 {
-        settings.geometry.h = 5;
+        settings.notification_limit = 5;
         settings.sort = true;
         settings.indicate_hidden = false;
         struct notification *nl1, *nl2, *nl3, *nl4, *nl5;
@@ -624,7 +752,7 @@ TEST test_queues_update_seeping(void)
         queues_notification_insert(nl5);
 
         QUEUE_LEN_ALL(5,0,0);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0,5,0);
 
         queues_notification_insert(nc1);
@@ -634,7 +762,7 @@ TEST test_queues_update_seeping(void)
         queues_notification_insert(nc5);
 
         QUEUE_LEN_ALL(5,5,0);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(5,5,0);
 
         QUEUE_CONTAINS(DISP, nc1);
@@ -656,7 +784,7 @@ TEST test_queues_update_seeping(void)
 TEST test_queues_update_xmore(void)
 {
         settings.indicate_hidden = true;
-        settings.geometry.h = 4;
+        settings.notification_limit = 4;
         struct notification *n1, *n2, *n3, *n4, *n5;
         queues_init();
 
@@ -670,15 +798,15 @@ TEST test_queues_update_xmore(void)
         queues_notification_insert(n2);
         queues_notification_insert(n3);
 
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0,3,0);
 
         queues_notification_insert(n4);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(0,4,0);
 
         queues_notification_insert(n5);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
         QUEUE_LEN_ALL(2,3,0);
 
         queues_teardown();
@@ -689,7 +817,7 @@ TEST test_queues_update_seep_showlowurg(void)
 {
         // Test 3 notifications during fullscreen and only the one
         // with the lowest priority is eligible to get shown
-        settings.geometry.h = 4;
+        settings.notification_limit = 4;
         struct notification *n1, *n2, *n3;
         queues_init();
 
@@ -705,12 +833,12 @@ TEST test_queues_update_seep_showlowurg(void)
 
         queues_notification_insert(n1);
         queues_notification_insert(n2);
-        queues_update(STATUS_FS);
+        queues_update(STATUS_FS, time_monotonic_now());
         QUEUE_LEN_ALL(2,0,0);
 
         queues_notification_insert(n3);
 
-        queues_update(STATUS_FS);
+        queues_update(STATUS_FS, time_monotonic_now());
 
         QUEUE_LEN_ALL(2,1,0);
         QUEUE_CONTAINS(WAIT, n1);
@@ -729,10 +857,10 @@ TEST test_queues_timeout_before_paused(void)
         n = test_notification("n", 10);
 
         queues_notification_insert(n);
-        queues_update(STATUS_NORMAL);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
 
         n->start -= S2US(11);
-        queues_update(STATUS_PAUSE);
+        queues_update(STATUS_PAUSE, time_monotonic_now());
 
         QUEUE_LEN_ALL(0,0,1);
 
@@ -763,9 +891,98 @@ TEST test_queue_find_by_id(void)
         PASS();
 }
 
+TEST test_queue_get_history(void)
+{
+        struct notification *n;
+        queues_init();
+
+        n = test_notification("n", -1);
+        n->skip_display = true;
+        queues_notification_insert(n);
+        n = test_notification("n1", -1);
+        n->skip_display = true;
+        queues_notification_insert(n);
+        n = test_notification("n3", -1);
+        n->skip_display = true;
+        queues_notification_insert(n);
+
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+
+        QUEUE_LEN_ALL(0, 0, 3);
+
+        GList *h = queues_get_history();
+        ASSERT(g_list_length(h) == 3);
+
+        queues_teardown();
+        PASS();
+}
+
+
+void print_queues() {
+        printf("\nQueues:\n");
+        for (GList *iter = g_queue_peek_head_link(QUEUE_WAIT); iter;
+                        iter = iter->next) {
+                struct notification *notif = iter->data;
+                printf("waiting %s\n", notif->summary);
+        }
+}
+
+// Test if notifications are correctly sorted, even if dunst is paused in
+// between. See #838 for the issue.
+TEST test_queue_no_sort_and_pause(void)
+{
+        // Setting sort to false, this means that notifications will only be
+        // sorted based on time
+        settings.sort = false;
+        settings.notification_limit = 0;
+        struct notification *n;
+        queues_init();
+
+        n = test_notification("n0", 0);
+        queues_notification_insert(n);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+
+        n = test_notification("n1", 0);
+        queues_notification_insert(n);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+
+        n = test_notification("n2", 0);
+        queues_notification_insert(n);
+        queues_update(STATUS_PAUSE, time_monotonic_now());
+
+        n = test_notification("n3", 0);
+        queues_notification_insert(n);
+        queues_update(STATUS_PAUSE, time_monotonic_now());
+        /* queues_update(STATUS_NORMAL, time_monotonic_now()); */
+
+        n = test_notification("n4", 0);
+        queues_notification_insert(n);
+        queues_update(STATUS_NORMAL, time_monotonic_now());
+
+        QUEUE_LEN_ALL(0, 5, 0);
+
+        const char* order[] = {
+                "n0",
+                "n1",
+                "n2",
+                "n3",
+                "n4",
+        };
+
+        for (int i = 0; i < g_queue_get_length(QUEUE_DISP); i++) {
+                struct notification *notif = g_queue_peek_nth(QUEUE_DISP, i);
+                ASSERTm("Notifications are not in the right order",
+                                STR_EQ(notif->summary, order[i]));
+        }
+
+        queues_teardown();
+        PASS();
+}
+
 SUITE(suite_queues)
 {
-        settings.icon_path = "";
+        bool store = settings.stack_duplicates;
+        settings.stack_duplicates = false;
 
         RUN_TEST(test_datachange_beginning_empty);
         RUN_TEST(test_datachange_endless);
@@ -783,8 +1000,11 @@ SUITE(suite_queues)
         RUN_TEST(test_queue_notification_close_histignore);
         RUN_TEST(test_queue_notification_skip_display);
         RUN_TEST(test_queue_notification_skip_display_redisplayed);
+        RUN_TEST(test_queue_notification_skip_display_redisplayed_by_random_id);
         RUN_TEST(test_queue_stacking);
         RUN_TEST(test_queue_stacktag);
+        RUN_TEST(test_queue_different_stacktag);
+        RUN_TEST(test_queue_stacktag_different_appid);
         RUN_TEST(test_queue_teardown);
         RUN_TEST(test_queue_timeout);
         RUN_TEST(test_queues_update_fullscreen);
@@ -794,8 +1014,10 @@ SUITE(suite_queues)
         RUN_TEST(test_queues_update_xmore);
         RUN_TEST(test_queues_timeout_before_paused);
         RUN_TEST(test_queue_find_by_id);
+        RUN_TEST(test_queue_no_sort_and_pause);
+        RUN_TEST(test_queue_get_history);
 
-        settings.icon_path = NULL;
+        settings.stack_duplicates = store;
 }
 
 /* vim: set tabstop=8 shiftwidth=8 expandtab textwidth=0: */
